# XCHK_AFM.afm (DOS/GAWK version)
#    usage: gawk -f xchk_afm.awk
#
# Check group of AFM files for different layout and each afm/pfb pair 
# for consistency (the same layout). AFM/PFB files should be present
# in the current directory.
#
# Note: T1DISASM shoud be present in your system while running this script!
#
# TP (april 5, 1996; april 28, 1996)
#

# CMP_FILES: compare afm/pfb layout
function cmp_files() {
   for (i=1; i<=pfb_char_num; i++)
        if ( pfb_char[i] != char[i] ) {return (1);  }
   return (0)
}

# CHK_PFB: extract characters from pfb file
function chk_pfb(fn, xc) {
   gsub (/\\AFM\\/,"\\", fn); # we are checking pfb file now;
   gsub (/\.AFM/,"\.PFB", fn); # we are checking pfb file now;
#   printf "%s ", fn  >> stdlog;
   
   system ("t1disasm " fn "\> disasm.tmp") #
   scan = 0;
   xc = 0;

   was_notdef=0
   while ( (getline pfb < "disasm.tmp" )  > 0) {
      
      if (pfb ~ /\/CharStrings/) {
         scan = 1; 
         print "Scanning pfb..." >> stderr;
         getline pfb < "disasm.tmp";
      } 
      else {
         if ( (scan == 1) && (pfb ~ /readonly put put/) ) {
            scan = 0; 
            print "done..." >> stderr;; 
            close ("disasm.tmp");
            return (xc);
         }
      }

      if (scan && pfb ~ /\.notdef/) was_notdef=1

      if (scan && (pfb ~ /^\//) && !(pfb ~ /\.notdef/)) {##
          xc++;
          gsub(/\//,"", pfb);
          split(pfb, tmp, " ");
          pfb_char[xc] = tmp[1];
          ### printf "" >> stderr; # slow down the scanning process
      }
   }

   # in case of any strange error we put:
   close ("disasm.tmp")
   return(-1);
}

# MAIN routine:
BEGIN { stdlog = "chk-afm.log" # standard log file
        stderr = "con" # standard error to console (for DOS)
        printf "" > stderr;  printf "" > stdlog; # delete 


   while (("dir /b /on /s ..\\*.afm" | getline  fname ) > 0) {
      scan = 0;    charno = 0;    FontName = "";
      files++;

      bfname = fname;
      gsub (/\.AFM/, "", bfname); # remove extension
      gsub (/.+\\/, "", bfname); # remove extension
      printf "\nIteration no:    %04d\n", files >> stderr;

      for (i in pfb_char) delete pfb_char[i]
      for (i in char) delete char[i]
      pfb_char_num = chk_pfb(fname);

      while ( (getline < fname )  > 0) {
         if ($1 == "FontName") {FontName = $2; }
         if ($1 == "StartCharMetrics") {scan = 1; getline < fname } else 
            {if ($1 ~ "EndCharMetrics") {scan = 0; break }}

         if (scan) {# ###
            charno++;
            char[charno] = $8;
         }
      }
     
      print "PS font name: ", FontName " (" bfname ")" >> stderr;

      printf "%-40s ", FontName >> stdlog;
      printf "%s ", bfname >> stdlog;
      printf "%3d ", charno >> stdlog; # chars found in afm file
      printf "%3d ", pfb_char_num >> stdlog; # chars found in pfb file
      printf " %s ", (was_notdef? "+" : "-" )  >> stdlog;

      if ( cmp_files() ) { 
        printf " * " >> stdlog; } ## Afm/pfb files are inconsistent!
      else { printf "  " >> stdlog; }

      # chk all layouts --------------------

      if (files==1) {# scanning first file --
         layout_num = 1;
#         fnlayouts[layout_num] = bfname;  # store file name 
         for (i=1; i <= charno; i++) { layouts[i,layout_num] = char[i] }
         printf "  " >> stdlog; 
         layout_curr=layout_num
      } 
      else {

         for (i=1; i<=layout_num; i++) {
             diff = 0; 
             for (j=1; j <= charno; j++) {
                 if (layouts[j,i] != char[j]) { diff++ }
                    # ##print layouts[j,i], char[j];
             }

             
             if (diff == 0) {# found identical
#                fnlayouts[i] = fnlayouts[i] ";" bfname; # store file name 
                layout_curr=i
                break; 
             }
         }


         if (diff) {## another layout!
            printf " * " >> stdlog;
            layout_num++;
            for (i=1; i <= charno; i++) { 
               layouts[i,layout_num] = char[i] 
            }

#            fnlayouts[layout_num] = bfname; # store file name 
            layout_curr=layout_num
         }   
         else { 
            printf "  " >> stdlog; 
         }

      }
     print " "  layout_curr >> stdlog; 

     layout_charno[layout_curr]=charno

     close (fname)

   }
#   close ("dir/b *.afm");
 
   # Print characters in all layouts

   printf "Scanned %d files\n", files >> stderr;
   printf "Found %s layout(s)\n", layout_num >> stderr;
   printf "inspect chk-afm.log file for details\n", layout_num >> stderr;

   # once again to log file (possible)

   printf "\nScanned %d files\n", files >> stdlog ;
   printf "Found %s layout(s)\n", layout_num >> stdlog;
#   printf " detailed statistics is as follows:\n" >> stdlog;

#   for (i=1; i<=layout_num; i++) {print " -- layout", i, "in: ", 
#        fnlayouts[i] >> stdlog;}

   for (i=1; i<=layout_num; i++) {
      printf "\nLayout %d\n======\n", i >> stdlog;
      clen = 0;

      for (j=1; j<=layout_charno[i]; j++) {
           printf "%s, ", layouts[j,i] >> stdlog;
           clen += (2 + length(layouts[j,i]));
           if (clen > 60) { clen = 0; printf "\n" >> stdlog; }
      }
      printf "\n" >> stdlog;
   }
   
}

